#################################################################
#                                                               #
#                          CLASSSES                             #
#                                                               #
#################################################################

'''
This file contains all the classes used in 
the Abaqus Winding Integration Tools

For more information abour thefunctions, how they work and why
the procedures have been cosen see 
"Integration Tools for Design and Process Control of Filament Winding"
by Inger Skjaerholt
'''

# necessary Python and Abaqus libraries and files
from abaqus import *
from abaqusConstants import *

import math

### global constants. Do. NOT. Change. These!
X_AXIS = 0
Y_AXIS = 1
Z_AXIS = 2

### BODY ###
### Filament winding machine limits
class MachineParameters:
	def __init__(self, string, maxMandrelRadius, maxWindingLength, maxMandrelLength, maxZStroke, maxWStroke):
		self.Name = string
		### machine parameters in mm
		# parameters on X-axis
		self.maxMandrelRadius = maxMandrelRadius
		self.maxWindingLength = maxWindingLength	#max length to wind
		
		# parameters on Y-axis
		self.maxMandrelLength = maxMandrelLength 	#max length including shafts

		self.maxZStroke = maxZStroke
		self.maxWStroke = maxWStroke


### cylindrical mandrel with spherical end domes and with identical dome openings
class CylindricalMandrel:
	def __init__(self, modelName, partName):
		# Model variables
		self.modelName = modelName
		self.partName = partName
		self.model = mdb.models[modelName]
		self.part = mdb.models[modelName].parts[partName]
		self.nodes = mdb.models[modelName].parts[partName].nodes
		self.elements = mdb.models[modelName].parts[partName].elements
		
		# Geometrical variables
		self.rotationalAxis = 'undetermined'
		self.H = None	#horizontal axis
		self.V = None	#vertical axis
		self.radius = None
		self.domeOpening = None
		self.minLength = None
		self.maxLength = None
		self.mandrelLength = None	#total length including domes
		self.cylinderLength = None	
		self.pointOfOrigin = None

		# G Code variables
		self.lines = list()
		self.domeVariable = 1
		self.windingDirection = -1
	
	### Class functions
	def setLength (self, minLength, maxLength):
		self.minLength = minLength
		self.maxLength = maxLength
		self.cylinderLength = (maxLength - minLength)
		
		self.calculateTotalLength()
		self.setDomeVariable()
		
	def calculateTotalLength(self):
		self.mandrelLength = 2* (math.sqrt(self.radius**2 - self.domeOpening**2) + 2*self.cylinderLength)

	def setDomeVariable(self):
		if (self.pointOfOrigin < self.minLength):
			self.domeVariable = -1
			
	# verifying that the mandrel does not exceed the capabilities of the winding machine
	def verifyMandrel (self, settings):
		
		if (self.radius > settings.maxMandrelRadius):
			print "The mandrel radius is too great for the ", settings.Name, " machine"
			
		if (self.mandrelLength > settings.maxWindingLength):
			print "The mandrel length is too great for the ", settings.Name, " machine"
		
		else:
			print "The mandrel is within the parameters of the ", settings.Name, " machine"
	
	def printProperties (self, settings):
		self.verifyMandrel(settings)
		print "Model: ", self.modelName, ", Part: ", self.partName
		print "Rotational Axis: ", self.rotationalAxis
		print "Radius: ", self.radius
		print "Dome Opening: ", self.domeOpening
		print "Length: ", self.cylinderLength


### common winding materials
class Material:
	def __init__(self):
		self.Name = None
		self.E_1 = None
		self.E_2 = None
		self.E_3 = None
		self.nu_12 = None
		self.nu_13 = None
		self.nu_23 = None
		self.G_12 = None
		self.G_13 = None
		self.G_23 = None
		self.rho = None
		self.thickness = None
		
	def carbon_epoxy(self):
		self.Name = 'Carbon_Epoxy'
		self.E_1 = 14000
		self.E_2 = 8000
		self.E_3 = 8000
		self.nu_12 = 0.3
		self.nu_13 = 0.3
		self.nu_23 = 0.55
		self.G_12 = 4000
		self.G_13 = 4000
		self.G_23 = 2581
		self.rho = 1600
		self.thickness = 0.15
		
		
### often used node variables
class CustomNode:
	def __init__(self, mandrel, element, node):
		# nodal coordinate values
		self.XYZ = [0, 0, 0]
		self.XYZ[X_AXIS] = mandrel.nodes[element.connectivity[node]].coordinates[X_AXIS]
		self.XYZ[Y_AXIS] = mandrel.nodes[element.connectivity[node]].coordinates[Y_AXIS]
		self.XYZ[Z_AXIS] = mandrel.nodes[element.connectivity[node]].coordinates[Z_AXIS]
		
		# Radii calculated using different coordinate combinations
		self.radii = [0, 0, 0]
		self.radii[X_AXIS] = self.calculateRadius(self.XYZ[Y_AXIS], self.XYZ[Z_AXIS])
		self.radii[Y_AXIS] = self.calculateRadius(self.XYZ[X_AXIS], self.XYZ[Z_AXIS])
		self.radii[Z_AXIS] = self.calculateRadius(self.XYZ[X_AXIS], self.XYZ[Y_AXIS])

	# calculate a radius using pythagoras
	def calculateRadius (self, leg1, leg2):
		return math.sqrt(leg1**2 + leg2**2)
		
	# get the node radius
	def returnRadius (self, rotationalAxis):
		return self.radii[rotationalAxis]
		
	# get the node coordinate ON the rotational axis 
	def returnAxisCoordinate(self, rotationalAxis):
		return self.XYZ[rotationalAxis]

	def printProperties (self, nodeName):
		print nodeName, ':'
		print 'XYZ = (', self.XYZ[0], ' , ', self.XYZ[1], ', ', self.XYZ[2], ')'
		print 'Radii = (', self.radii[0], ' , ', self.radii[1], ', ', self.radii[2], ')'

### CNC code line to be put in the mandrel variable 'lines'
class CNCLine:
	def __init__(self, N, X, Y, Z, W):
		self.N = N
		self.X = X
		self.Y = Y
		self.Z = Z
		self.W = W
		
		self.string = "N{0} X{1} Y{2} Z{3} W{4}\n".format(
			str(round(N, 5)), str(round(X, 5)), str(round(Y, 5)), str(round(Z, 5)), str(round(W, 5)))
		
		
### G code variables
class GCode:
	def __init__(self, filename, mandrel):
		
		# increment variables
		self.N = 10			#variable for the sequence number
		self.increment = 10	#sequence number incremental value
		
		# movement variables
		self.totalX = 0
		self.totalY = 0
		self.totalZ = 0
		self.totalW = 0
		
		self.writeOpeningLines(filename, mandrel)
		
	def writeOpeningLines(self, filename, mandrel):
		self.GCodeFile = open(filename, 'w')
		
		### as the G-code file is created the topmost lines in the program are written
		# comment lines to explain for what type of mandrel the program has been written
		self.GCodeFile.write ('(This CNC program has been created using)\n')
		self.GCodeFile.write ('(The Abaqus Winding Integration Tools)\n')
		self.GCodeFile.write ('for a mandrel of:)\n')
		self.GCodeFile.write ('(\tCylinder Radius: ' + str(round(mandrel.radius, 2)) + ')\n')
		self.GCodeFile.write ('(\tDome Opening: ' + str(round(mandrel.domeOpening, 2)) + ')\n')
		self.GCodeFile.write ('(\tCylinder Length: ' + str(round(mandrel.cylinderLength, 2)) + ')\n')
		self.GCodeFile.write ('\n\n\n')
		
		# first block of the program with preparatory commands etc.
		self.GCodeFile.write ('N10 G01 G21 G91 G94 F50000\n')

	def addLine (self, settings, mandrel, X, Y, Z, W):
		
		### adding the movement to the total movement along the axes
		self.totalX += X
		self.totalY += Y
		self.totalZ += Z
		self.totalW += W
		
		### ensuring that the limits of the machine have not been exceeded
		if (self.totalY > settings.maxMandrelLength):
			print "error! The maximum movement along the X-axis has been exceeded"
		
		if (self.totalZ > settings.maxZStroke):
			print "error! The maximum movement along the Y-axis has been exceeded"
		
		if (self.totalW > settings.maxWStroke):
			print "error! The maximum rotation around th W-axis has been exceeded"
		
		else:
			# constructing the next line
			self.N += self.increment
			line = CNCLine (self.N, X, Y, Z, W)
			
			# adding the line to the list of program lines
			mandrel.lines.append(line)
			
			# writing the line into the program file
			self.GCodeFile.write(line.string)
			return
		
		# the limits of the machine have been exceeded 
		# the process is stopped
		return "something to stop the loop"


### variables used while adding layup to a model
class LayupConstruction:
	def __init__(self, P1):
		# general variables
		self.material = Material()
		
		# movement variables
		self.P1 = P1			#starting point for CNC block
		self.P2 = [0, 0, 0]		#ending point for CNC block
		self.crossesHorizontal = False
		self.crossesVertical = False

	def printPoints(self):
		print 'P1: (', self.P1[0], ', ', self.P1[1], ', ', self.P1[2], ')'
		print 'P2: (', self.P2[0], ', ', self.P2[1], ', ', self.P2[2], ')'


### Assembly with feed eye
class Assembly:
	def __init__(self, mandrel):
		self.feedEye = 'Feed-Eye'
		self.mandrelName = mandrel.modelName
		self.assemblyDir = None
		
		self.createFeedEye(mandrel)
		
	def createFeedEye(self, mandrel):
		# base sketch
		sketch = mandrel.model.ConstrainedSketch (name = 'Section Sketch', sheetSize = 200.0)

		sketch.CircleByCenterPerimeter((0.0, 0.0), (0.0, 5.0))
		
		# creating the part
		part = mandrel.model.Part(name = self.feedEye)
		part.BaseSolidExtrude (sketch = sketch, depth = 5.0)
		
	def move (self, mandrel, Y, Z):
		#using the axes of the FW machine at NTNU as a standard
		# X = rotation of mandrel
		# Y = movement along the rotational axis
		# Z = movement perpendicular to the mandrel
		# W = rotation of the feed eye 
		
		# currently W and X are not relevant, but they will probably be in the future
		# e.g. in the case of a non-axisymmetric mandrel

		newCoords = [0, 0, 0]
		newCoords[mandrel.rotationalAxis] = Y
		newCoords[mandrel.V] = -1*Z

		self.assemblyDir.translate((self.feedEye, ), newCoords)
		session.viewports['Viewport: 1'].view.setValues(drawImmediately = True)

