#################################################################
#                                                               #
#                   FUNCTIONS TO ADD LAYUP                      #
#                                                               #
#################################################################

'''
This file contains all the custom made functions necessary to  
add layup on an Abaqus model

For more information about the functions, how they work and why
the procedures have been chosen see 
"Integration Tools for Design and Process Control of Filament Winding"
by Inger Skjaerholt

'''

# necessary Python, Abaqus libraries and files
from abaqus import *
from abaqusConstants import *
import __main__
import regionToolset

import re		#necessary library to search through a string
import math

import classes
reload (classes)

### global constants. Do. NOT. Change. These!
# (global constants added for readability)
X_AXIS = 0
Y_AXIS = 1
Z_AXIS = 2

### BODY ###
'''
This function adds the carbon_epoxy material to the model
'''
def addMaterial(mandrel, layup):
	layup.material.carbon_epoxy()
	
	# add material to the Abaqus model
	mdb.models[mandrel.modelName].Material(name = layup.material.Name)

	# set material properties
	mdb.models[mandrel.modelName].materials[layup.material.Name].Elastic(
		type = ENGINEERING_CONSTANTS, 
		table = ((layup.material.E_1, layup.material.E_2, layup.material.E_3, 
			layup.material.nu_12, layup.material.nu_13, layup.material.nu_23, 
			layup.material.G_12, layup.material.G_13, layup.material.G_23), ))
	
	# set material density
	mdb.models[mandrel.modelName].materials[layup.material.Name].Density(table=((layup.material.rho, ), ))


'''
This function extracts the relevant values from a line of G code
'''
def readGCodes(line):	
	try:
		matches = re.search(r'\A N(\d+) \s+ X(\d+(,\d+)?) \s+ Y(\d+(,\d+)?)', line, flags=re.M | re.S | re.X)
		
		X = float(matches.group(2).replace(",", "."))
		Y = float(matches.group(4).replace(",", "."))
		
		return (X, Y)

	except AttributeError:
		return


'''
This function to determine if a variable is positive or negative
'''
def sign(var):
	if var < 0:
		return -1
	return 1


'''
This function determines the coordinate values for P2 
based on P1, rotation of mandrel and movement along the Y-axis
'''
def calculateP2(mandrel, layup, X, Y):	
	P1 = layup.P1
	
	# determine what quadrant P1 is located in
	signH = sign(P1[mandrel.H])
	signV = sign(P1[mandrel.V])
	
	# calculate the angle between P1 and the horizontal axis
	U = math.atan2(abs(P1[mandrel.V]),abs(P1[mandrel.H]))

	# if the coordinates of P1 are in quadrant I or III
	if (signH == signV):
		# if P1 and P2 are in the same quadrant
		if (U+X) < (math.pi/2):
			W = U+X	
		
		else:
			W = math.pi - (U+X)
			signH *= -1
			layup.crossesVertical = True
	
	# if the coordinates of P1 are in quadrant II or IV
	else:
		# if P1 and P2 are in the same quadrant
		if (X < U):
			W = U-X
		
		else:
			W = X-U
			signV *= -1
			layup.crossesHorizontal = True
			
	legH = signH * (mandrel.radius * math.cos(W))		#horizontal leg 
	legV = signV * (mandrel.radius * math.sin(W))		#vertical leg
	legLateral = P1[mandrel.rotationalAxis] + mandrel.windingDirection * Y
	
	### set values for P2
	layup.P2[mandrel.rotationalAxis] = legLateral
	layup.P2[mandrel.V] = legV
	layup.P2[mandrel.H] = legH
	

'''
This function collects the elements between P1 and P2
'''
def collectBoxElements(mandrel, layup):
	minCoords = [0, 0, 0]	#minimum coordinates
	maxCoords = [0, 0, 0]	#maximum coordinates
	P1 = layup.P1
	P2 = layup.P2

	# sort maximum and minimum coordinates
	for i in range(3):
		if (P1[i] < P2[i]):
			minCoords[i] = P1[i]
			maxCoords[i] = P2[i]
		else:
			minCoords[i] = P2[i]
			maxCoords[i] = P1[i]

	# if an axis has been crossed the max/min variable 
	# must be mandrel radius to collect all elements
	if layup.crossesHorizontal:
		if sign(P1[mandrel.V]) >  0:
			minCoords[mandrel.H] = -1* mandrel.radius
		else:
			maxCoords[mandrel.H] = mandrel.radius

	if layup.crossesVertical:
		if sign(P1[mandrel.H]) > 0:
			maxCoords[mandrel.V] = mandrel.radius
		else:
			minCoords[mandrel.V] = -1*mandrel.radius

	boxElements = mandrel.elements.getByBoundingBox(
		xMin = minCoords[X_AXIS],
		yMin = minCoords[Y_AXIS],
		zMin = minCoords[Z_AXIS],
		xMax = maxCoords[X_AXIS],
		yMax = maxCoords[Y_AXIS],
		zMax = maxCoords[Z_AXIS])

	return boxElements


'''
This function determines which elements are crossed by the line between P1 and P2
'''
def collectLayupElements(mandrel, layup, boxElements):
	P1 = layup.P1
	P2 = layup.P2
	
	### determine the line equation for the line between P1 and P2
	# if P1 is in quadrant I or III
	if sign(P1[mandrel.H]) == sign(P1[mandrel.V]):
		a = ((P2[mandrel.rotationalAxis] - P1[mandrel.rotationalAxis])/(P2[mandrel.H] - P1[mandrel.H]))
		b = P1[mandrel.rotationalAxis] - P1[mandrel.H]*a
	
	# if P1 is in quadrant II or IV
	elif sign(P1[mandrel.H]) != sign(P1[mandrel.V]):
		a = ((P2[mandrel.rotationalAxis] - P1[mandrel.rotationalAxis])/(P2[mandrel.V] - P1[mandrel.V]))
		b = P1[mandrel.rotationalAxis] - P1[mandrel.V]*a
	
	sectionElements = []

	for element in boxElements:
		numberOfNodes = len(element.connectivity)
		
		# iterate through the nodes 
		for i in range(numberOfNodes):
			currentNode = classes.CustomNode(mandrel, element, i)
			
			# set points of comparison
			if i == 0:
				minY = maxY = currentNode.XYZ[mandrel.rotationalAxis]
				minX = maxX = currentNode.XYZ[mandrel.H]
				
				# for quadrant II and IV use vertical axis
				if sign(P1[mandrel.H]) != sign(P1[mandrel.V]):
					minX = maxX = currentNode.XYZ[mandrel.V]
						
			else:
				if (minY > currentNode.XYZ[mandrel.rotationalAxis]):
					minY = currentNode.XYZ[mandrel.rotationalAxis]

				if (maxY < currentNode.XYZ[mandrel.rotationalAxis]):
					maxY = currentNode.XYZ[mandrel.rotationalAxis]
				
				# for quadrant II and IV use vertical axis
				if sign(P1[mandrel.H]) != sign(P1[mandrel.V]):
					if (minX > currentNode.XYZ[mandrel.V]):
						minX = currentNode.XYZ[mandrel.V]
						
					if (maxX < currentNode.XYZ[mandrel.V]):	
						maxX = currentNode.XYZ[mandrel.V]
				
				# for quadrant I and III use horizontal axis
				else:
					if (minX > currentNode.XYZ[mandrel.H]):
						minX = currentNode.XYZ[mandrel.H]	

					if (maxX < currentNode.XYZ[mandrel.H]):
						maxX = currentNode.XYZ[mandrel.H]
		
		### calculating coordinates for a point on the line
		  # corresponding to the min/max values of the element
		Y_min = a*minX + b
		Y_max = a*maxX + b
		X_min = (minY - b)/a
		X_max = (maxY - b)/a
		
		### checking if the element is crossed by the line between P1 and P2
		appended = False 	#variable ensures an element is only appended once
		if (((minY < Y_min) and (Y_min < maxY)) or ((minY < Y_max) and (Y_max < maxY))):
			appended = True
			sectionElements.append(element)

		if (((minX < X_min) and (X_min < maxX)) or ((minX < X_max) and (X_max < maxX))):
			if appended == False:
				sectionElements.append(element)

	return sectionElements


'''
This function adds ply to the layup of each relevant element
'''
def addPly (mandrel, layup, sectionElements, X, Y):
	rotAxes = (AXIS_1, AXIS_2, AXIS_3)
	alpha = math.degrees(math.atan(Y/mandrel.radius/X)) 	#winding angle based on machine movement

	for element in sectionElements:
		layupName = 'layup e[%r]' %element.label
		
		#check if the current element already has a composite layup
		try:
			compositeLayup = mandrel.part.compositeLayups[layupName]
			
			numPlies = len(compositeLayup.plies)+1
			plyName = 'Ply %r' %numPlies
		
		# if not: create composite layup for the element
		except KeyError:
			# create composite layup
			compositeLayup = mandrel.part.CompositeLayup(
				name = layupName,
				offsetType = TOP_SURFACE,
				symmetric = False,
				thicknessAssignment = FROM_SECTION)
			
			plyName = 'Ply 1'
			
			# set rotation to be relative to the global coordinate system
			compositeLayup.orientation.setValues(
				orientationType = GLOBAL,
				localCsys = None,
				additionalRotationType = ROTATION_NONE, 
				angle = 0.0)
		
		# format the element to be put into the region
		temp = mandrel.elements.sequenceFromLabels([element.label])
		
		region1 = regionToolset.Region(elements = temp)
		
		# add ply to element
		compositeLayup.CompositePly(
			suppressed = False, 
			plyName = plyName,
			thicknessType = SPECIFY_THICKNESS,
			thickness = layup.material.thickness,
			region = region1,
			material = layup.material.Name,
			orientationType = SPECIFY_ORIENT,
			orientationValue = alpha,
			axis = rotAxes[mandrel.rotationalAxis])


'''
This function adds layup to a mandrel model based on: 
	- CNC program from file 'f'
	- Abaqus model that has generated its own CNC program 'g'
	- manual input 'm'
'''
def addLayup (mandrel, method):
	# CNC program from file
	if method == 'f':
		fileName = getInput('Enter the file name of the CNC-program:')
		str(fileName)

		file = open(fileName, 'r')

		### set P1 as tip of dome	
		P1 = [0, 0, 0]
		P1[mandrel.rotationalAxis] = mandrel.pointOfOrigin
		
		# create layup variables
		layup = classes.LayupConstruction(P1)
		addMaterial(mandrel,layup)
		
		for line in file:
			XY = readGCodes(line)
			# if XY is empty the current line contains comments or modal commands
			if not XY:
				continue
			
			# change winding direction if appropriate
			if (XY[1] < 0):
				mandrel.windingDirection = 1
			
			### add layup based on variables from file
			calculateP2(mandrel, layup, XY[0], XY[1])
			boxElements = collectBoxElements(mandrel, layup)
			sectionElements = collectLayupElements(mandrel, layup, boxElements)
			addPly(mandrel, layup, sectionElements, XY[0], XY[1])

			P1 = P2
	
	# layup created based on relevant abaqus model
	elif method == 'g':
		### set P1 as dome tip
		P1 = [0, 0, 0]
		P1[mandrel.rotationalAxis] = mandrel.pointOfOrigin
		
		# create layup variables
		layup = classes.LayupConstruction(P1)
		addMaterial(mandrel,layup)
		
		for i in mandrel.lines:
			line = mandrel.lines[i]
			X = line.X
			Y = line.Y
			
			#change winding direction if appropriate
			if (Y < 0):
				mandrel.windingDirection = 1
			
			### add layup based on variable from the 'lines' list of the mandrel
			calculateP2(mandrel, layup, X, Y)
			boxElements = collectBoxElements(mandrel, layup)
			sectionElements = collectLayupElements(mandrel, layup, boxElements)
			addPly(mandrel, layup, sectionElements, X, Y)

			P1 = P2

	# manual input, mostly used for testing and debugging
	elif method == 'm':
		startingPoints = (
			[25, 20, 15], 
			[25, 20, -15], 
			[25, -20, -15], 
			[25, -20, 15])

		XY = (3, 40)
		
		for i in range(len(startingPoints)):
			layup = classes.LayupConstruction(startingPoints[i])
			addMaterial(mandrel,layup)

			calculateP2(mandrel, layup, XY[0], XY[1])
			boxElements = collectBoxElements(mandrel, layup)
			sectionElements = collectLayupElements(mandrel, layup, boxElements)
			addPly(mandrel, layup, sectionElements, XY[0], XY[1])














