#################################################################
#                                                               #
#          FUNCTIONS TO DETERMINE MANDREL PROPERTIES            #
#                                                               #
#################################################################

'''
This file contains functions to gather and set mandrel properties
based on a model from Abaqus/CAE

For more information about the functions, how they work and why
the procedures have been chosen see 
"Integration Tools for Design and Process Control of Filament Winding"
by Inger Skjaerholt

'''

# necessary Python, Abaqus libraries and files
from abaqus import *

import classes
reload (classes)

### global constants. Do. NOT. Change. These!
X_AXIS = 0
Y_AXIS = 1
Z_AXIS = 2

### BODY ###
'''
This function iterates through the list of mandrel elements
comparing radii in different directions to determine Rotational Axis
'''
def determineRotationalAxis (mandrel):
	counter = 0	
	element = mandrel.elements[counter]			#first element in the elements list
	test = None		#fail-safe test variable
	
	currentNode = classes.CustomNode(mandrel, element, 0)	

	while mandrel.rotationalAxis == 'undetermined':
		counter += 1

		# if the counter has moved to the end of the elements list
		# it was not possible to determine the rotational axis
		if counter >= len(mandrel.elements):
			print "it was not possible to determine the rotational axis"
			
			#ask for manual input through GUI
			rotationalAxis = getInput('Enter the numerical value for the rotational Axis: \n(X-Axis = 0, Y-Axis = 1, Z-Axis = 2)')
			mandrel.rotationalAxis = int(rotationalAxis)
			break
	
		element = mandrel.elements[counter]		#next element
		nextNode = classes.CustomNode(mandrel, element, 0)

		# if the coordinates are identical it is unnecessary to compare them
		if (currentNode.XYZ == nextNode.XYZ):
			currentNode = nextNode
			continue
		
		### comparing to see if the different radii are equal, determining the rotational axis
		if round(currentNode.radii[X_AXIS],5) == round(nextNode.radii[X_AXIS], 5):
			# if the fail-safe variable is 'None' or another axis
			if test != X_AXIS:
				test = X_AXIS
			
			# if the previous comparison also returned this axis, the rotational axis is set
			elif test == X_AXIS:	
				mandrel.rotationalAxis = X_AXIS
				mandrel.H = Z_AXIS
				mandrel.V = Y_AXIS

		if round(currentNode.radii[Y_AXIS], 5) == round(nextNode.radii[Y_AXIS], 5):
			if test != Y_AXIS:
				test = Y_AXIS
			
			elif test == Y_AXIS:
				mandrel.rotationalAxis = Y_AXIS
				mandrel.H = X_AXIS
				mandrel.V = Z_AXIS
			
		if round(currentNode.radii[Z_AXIS], 5) == round(nextNode.radii[Z_AXIS], 5):
			if test != Z_AXIS:
				test = Z_AXIS
			
			elif test == Z_AXIS:
				mandrel.rotationalAxis = Z_AXIS
				mandrel.H = Y_AXIS
				mandrel.V = X_AXIS


		# if the rotational axis has not been determined move one element further
		currentNode = nextNode


'''
This function iterates through all the nodes on all the elements in the elements list
determining the smallest and greatest radius along the axis of rotation.
'''
def determineRadii (mandrel): 
	minRadius = None
	maxRadius = None
	
	for element in mandrel.elements:
		nodesOnElement = len(element.connectivity) 		#number of nodes on an element. Will vary depending on element type

		# iterate through the nodes
		for i in range(nodesOnElement):
			currentNode = classes.CustomNode(mandrel, element, i)
			tempRadius = currentNode.returnRadius(mandrel.rotationalAxis)			# radius given by node j on element i 
			
			# set point of comparison on the first iteration
			if minRadius == None:
				minRadius = tempRadius
				maxRadius = tempRadius
				
				# in case the first node is on the dome opening
				mandrel.pointOfOrigin = currentNode.returnAxisCoordinate(mandrel.rotationalAxis)
				
			elif tempRadius < minRadius:
				minRadius = tempRadius
				mandrel.pointOfOrigin = currentNode.returnAxisCoordinate(mandrel.rotationalAxis)

			
			elif tempRadius > maxRadius:
				maxRadius = tempRadius
	
	mandrel.radius = maxRadius
	mandrel.domeOpening = minRadius
	
	
'''
This function iterates through all the elements in the elements list
determining the outer extreme coordinates on the rotational axis for the cylinder
'''
def determineLength (mandrel):
	minLength = None			#minimum coordinate value on the rotational axis
	maxLength = None			#maximum coordinate value on the rotational axis

	for element in mandrel.elements:
		nodesOnElement = len(element.connectivity)		# variable for the number of nodes on an element. Will vary depending on element type 

		# iterate through the nodes
		for i in range(nodesOnElement):
			currentNode = classes.CustomNode(mandrel, element, i)
			rotAxCoordinate = currentNode.returnAxisCoordinate(mandrel.rotationalAxis)		
			tempRadius = currentNode.returnRadius(mandrel.rotationalAxis)

			# tempRadius is not equal to the cylinder radius and therefore on the dome
			if round(tempRadius, 5) != round(mandrel.radius, 5):
				break
			
			# set points of comparison on the first iteration
			if minLength == None:
				minLength = rotAxCoordinate
				maxLength = rotAxCoordinate
			
			elif rotAxCoordinate < minLength:
				minLength = rotAxCoordinate
			
			elif rotAxCoordinate > maxLength:
				maxLength = rotAxCoordinate

	mandrel.setLength(minLength, maxLength)

	
'''
This function gathers and determines the mandrel properties 
based on a specific model in Abaqus/CAE
'''
def setProperties(settings, mandrel):
	determineRotationalAxis(mandrel)
	determineRadii(mandrel)
	determineLength(mandrel)

	print 'Mandrel Properties: '
	print '-----------------------------------------'
	mandrel.printProperties(settings)




