#################################################################
#                                                               #
#             FUNCTIONS TO RUN A VISUAL CRASH TEST              #
#                                                               #
#################################################################

'''
This file contains all the custom made functions necessary to run a 
visual crash test using the Abaqus model.

For more information about the functions, how they work and why
the procedures have been chosen see 
"Integration Tools for Design and Process Control of Filament Winding"
by Inger Skjaerholt
'''

# necessary Python, Abaqus libraries and files
from abaqus import *
from abaqusConstants import *
import __main__

import sketch
import part

import time

import classes
reload (classes)

### global constants. Do. NOT. Change. These!
# constants representing coordinate system values (either axes or coordinates):
X_AXIS = 0
Y_AXIS = 1
Z_AXIS = 2

### BODY ###
'''
This function creates an assembly with a constructed feed eye and the relevant part
'''
def createTestSetUp (mandrel, assembly):
	# creating assembly
	assembly.assemblyDir = mandrel.model.rootAssembly
	
	# display the assembly in the viewport
	session.viewports['Viewport: 1'].setValues(displayedObject=assembly.assemblyDir)

	feedEyePart = mandrel.model.parts[assembly.feedEye]
	mandrelPart = mandrel.model.parts[mandrel.partName]

	#adding the two parts to the assembly
	assembly.assemblyDir.Instance (assembly.feedEye, part = feedEyePart, dependent = ON)
	assembly.assemblyDir.Instance ('Mandrel', part = mandrelPart, dependent = ON)

	# define practical view based on rotational axis
	# and rotate feed eye according to model
	if mandrel.rotationalAxis == X_AXIS:
		assembly.assemblyDir.rotate((assembly.feedEye, ), (0,0,0), (0,0,-5), 90)
		setView = (45, 45, 0)
	
	if mandrel.rotationalAxis == Y_AXIS:
		assembly.assemblyDir.rotate((assembly.feedEye, ), (0,0,0), (0,-5,0), 90)
		setView = (-45, 0, -45)
	
	if mandrel.rotationalAxis == Z_AXIS:
		assembly.assemblyDir.rotate((assembly.feedEye, ), (0,0,0), (-5,0,0), 90)
		setView = (45, 135, 90)

	### move the feed eye to an initial position
	offset = [0, 0, 0]
	offset[mandrel.H] = mandrel.radius + mandrel.radius/4
	offset[mandrel.rotationalAxis] = mandrel.pointOfOrigin
	assembly.assemblyDir.translate((assembly.feedEye, ), offset)
	
	# set appropriate view and fit to screen
	session.viewports['Viewport: 1'].view.rotate(xAngle=setView[0], yAngle=setView[1], zAngle=setView[2], mode=TOTAL)
	session.viewports['Viewport: 1'].view.fitView()


'''
This function runs a visual crash test based on generated CNC program for the model
'''
def runTest (mandrel):
	assembly = classes.Assembly(mandrel)
	createTestSetUp(mandrel, assembly)

	for i in range(len(mandrel.lines)):
		assembly.move(mandrel, mandrel.lines[i].Y, mandrel.lines[i].Z)
		time.sleep(1)
